function [dataSet, setName] = LoadEVBSet
%LOADEVBSET  - load trajectory dataset (EVB format)
%
% 	usage:  [dataSet, setName] = LoadEVBSet
%
% assumes that MAT files have the following organization:
%
%	<SUBJID>_<DATATYPE>_<DATAID>
%
% where
%	SUBJID specifies the subject
%	DATATYPE is one of M (movement), S (speech), V (via-point data)
%	DATAID identifies the data (e.g. "sta" = stationary)
%
% returns DATASET, an array of structs with fields
%	NAME	- trajectory name
%	SRATE	- sampling rate (Hz)
%	SIGNAL	- signal data [nSamp (samples) x nComp (components)]
%	SCALING	- monodimensional data are UNIQUE, multi are COMMON
%
% if via-point data is loaded, creates and fills the LABELS field
% corresponding to (VIA.codex) labelled VPs.

% --- Working assumptions:
%
% signal data is in a field called 'signal' organized such that
%	each row gives data for one dimension of one trajectory
%
% sampling rate is given by a field called 'srate' in Hz
%
% trajectory names are given by a field called 'chan' ordered to
%	match signal data; length(chan) == length(signal,1)/(nDimensions)
%
% VP data is given by a struct called 'codex' with fields
% 	'time', [triphone sample offsets (nVP x 3)], 'tri', [triphone label]
%	time values are offset by the contents of the field called 'ends'

% 10/26/98  mkt

%	constants

mString = '_m_';			% movement data file identifier
sString = '_s_';			% speech data file identifier
vString = '_v_';			% VP data file identifier
eString = '_e_';			% EMG data file identifier

mVarName = 'MVT_data';		% movement variable name
sVarName = 'SPC_data';		% speech variable name
vVarName = 'VIA';			% VP variable name
eVarName = 'EMG_data';		% EMG variable name

mask = ['*' mString '*'];	% file selection mask

dataSet = [];

%	get movement filename

[fileName, pathName] = uigetfile(mask, 'Select EVB format movement file');
if fileName == 0, return; end;	% cancelled


%	build filenames

i = findstr(fileName, mString);
subjID = fileName(1:i-1);					% subject ID
j = findstr(fileName, '.');
dataID = fileName(i+length(mString):j-1);	% data ID
ext = fileName(j:end);						% extension
mFile = fileName;							% movement filename
sFile = [subjID sString dataID ext];		% speech filename
if exist([pathName sFile]) ~= 2,
	sFile = [];								% no speech file
end;
vFile = [subjID vString dataID ext];		% VP filename
if exist([pathName vFile]) ~= 2,
	vFile = [];								% no VP file
end;
eFile = [subjID eString dataID ext];		% EMG filename
if exist([pathName eFile]) ~= 2,
	eFile = [];								% no EMG file
end;


%	load the movement data

vars = who('-file', [pathName mFile]);
if ~strmatch(mVarName, char(vars), 'exact'),
	set(warndlg([mFile ' does not contain a variable called ' mVarName], ''), 'WindowStyle', 'modal');
	return;
end;

load([pathName mFile], mVarName);
eval(['numIndices = length(' mVarName ');']);
choix = Select(numIndices, ~isempty(sFile), ~isempty(vFile), ~isempty(eFile));
if isempty(choix), return; end;		% cancelled
eval(['mData = ' mVarName '(choix.INDEX);']);	% extract selected token
eval(['clear ' mVarName]);						% conserve memory


%	load the speech data

if choix.SPEECH,
	vars = who('-file', [pathName sFile]);
	if ~strmatch(sVarName, char(vars), 'exact'),
		set(warndlg([sFile ' does not contain a variable called ' sVarName], ''), 'WindowStyle', 'modal');
		return;
	end;
	
	load([pathName sFile], sVarName);
	eval(['sData = ' sVarName '(choix.INDEX);']);	% extract selected token
	eval(['clear ' sVarName]);						% conserve memory
else,
	sData = [];
end;

%	load the VP data

if choix.VPS,
	vars = who('-file', [pathName vFile]);
	if ~strmatch(vVarName, char(vars), 'exact'),
		set(warndlg([vFile ' does not contain a variable called ' vVarName], ''), 'WindowStyle', 'modal');
		return;
	end;
	
	load([pathName vFile], vVarName);
	eval(['vData = ' vVarName '(choix.INDEX);']);		% extract selected token
	vpOffset = vData.ends(1);							% offset of 1st VP
	vData = vData.codex;								% keep codex
	eval(['clear ' vVarName]);							% conserve memory
else,
	vData = [];
end;

%	load the EMG data

if choix.EMG,
	vars = who('-file', [pathName eFile]);
	if ~strmatch(eVarName, char(vars), 'exact'),
		set(warndlg([eFile ' does not contain a variable called ' eVarName], ''), 'WindowStyle', 'modal');
		return;
	end;
	
	load([pathName eFile], eVarName);
	eval(['eData = ' eVarName '(choix.INDEX);']);		% extract selected token
	eval(['clear ' eVarName]);							% conserve memory
else,
	eData = [];
end;

%	assemble the dataset

if ~isempty(sData),
	range = ceil(max(abs([min(sData.signal) max(sData.signal)])));
	dataSet = struct('NAME', 'audio', ...
					'SRATE', sData.srate, ...
					'SIGNAL', sData.signal', ...
					'SCALING', struct('TYPE', 'UNIQUE', 'RANGE', [-range range]), ...
					'LABELS', []);	
	clear sData;
end;

if ~isempty(eData),
	for t = 1 : size(eData.chan,1),
		range = ceil(max(abs([min(eData.signal(t,:)) max(eData.signal(t,:))])));
		element = struct('NAME', char(eData.chan(t)), ...
						'SRATE', eData.srate, ...
						'SIGNAL', eData.signal(t,:)', ...
						'SCALING', struct('TYPE', 'UNIQUE', 'RANGE', [-range range]), ...
						'LABELS', []);
		if isempty(dataSet),
			dataSet = element;
		else,
			dataSet(end+1) = element;
		end;
	end;
end;

nTraj = size(mData.chan,1);			% number of trajectories
nComp = size(mData.signal,1)/nTraj;	% number of dimensions per trajectory
for t = 1 : nTraj,
	i = (t-1)*nComp + [1 2 3];
	element = struct('NAME', char(mData.chan(t)), ...
					'SRATE', mData.srate, ...
					'SIGNAL', mData.signal(i,:)', ...
					'SCALING', struct('TYPE', 'COMMON', 'RANGE', []), ...
					'LABELS', []);
	element.SCALING.RANGE = [min(element.SIGNAL)' max(element.SIGNAL)'];	
	if isempty(dataSet),
		dataSet = element;
	else,
		dataSet(end+1) = element;
	end;
end;

if ~isempty(vData),
	for t = 1 : length(dataSet),
		for i = 1 : size(vData.time,1),
			offset = 1000*(vData.time(i,:)+vpOffset-1)/mData.srate;		% VP triplet offsets in msecs
			if t == 1, name = char(vData.tri(i)); else, name = ''; end;	% avoid visual clutter -- only 1st trajectory gets names
			label = struct('NAME', name, ...
							'OFFSET', offset(2), ...		% triphone center
							'NOTE', offset, ...				% save all three as data
							'VALUE', []);					% don't worry about values for now
			if isempty(dataSet(t).LABELS),
				dataSet(t).LABELS = label;
			else,
				dataSet(t).LABELS(end+1) = label;
			end;
		end;
	end;
end;

%	name the dataset

setName = [subjID '_' dataID];


%=============================================================================
% SELECT  - return index of selected token
%
% args:
%	NUMINDICES	- number of tokens available
%	GOTSPEECH	- speech data available
%	GOTVPS		- VP data available
%
% returned structure is
%	INDEX	- selected index
%	SPEECH	- load speech if non-zero
%	VPS		- load VPs if non-zero
%
% returns [] on cancel

function choix = Select(numIndices, gotSpeech, gotVPs, gotEMG)

figPos = get(0, 'ScreenSize');

width = 200;
height = 150;
figPos = [figPos(1)+(figPos(3)-width)/2, figPos(2)+(figPos(4)-height)/2, width, height];
if strncmp(computer, 'SGI', 3), as = 'left'; else, as = 'right'; end;	% SGI bug:  doesn't like right alignment

cfg = dialog('Name', 'Select token...', ...
	'Position', figPos, ...
	'ButtonDownFcn', '', ...
	'UserData', 0);

% token index list
list = uicontrol(cfg, ...
	'Position',[10 height-90 80 80], ...
	'String', num2str([1:numIndices]'), ...	
	'ListBoxTop', 1, ...
	'Style','listbox', ...
	'Value', 1, ...
	'Callback', 'if strcmp(get(gcbf,''SelectionType''),''open''),set(gcbf,''UserData'',1);uiresume;end');

% text label
uicontrol(cfg, ...
	'Position', [110 height-30 35 17], ...
	'Style', 'text', ...
	'HorizontalAlignment', 'left', ...
	'String', 'Load');

% load speech CB
if gotSpeech, s = 'on'; else, s = 'off'; end;
speech = uicontrol(cfg, ...
	'Position', [120 height-45 68 20], ...
	'Style', 'checkbox', ...
	'String', 'speech', ...
	'Enable', s, ...
	'Value', gotSpeech);
	
% load VPs CB
if gotVPs, s = 'on'; else, s = 'off'; end;
vps = uicontrol(cfg, ...
	'Position', [120 height-65 50 20], ...
	'Style', 'checkbox', ...
	'String', 'VPs', ...
	'Enable', s, ...
	'Value', gotVPs);
	
% load EMG CB
if gotEMG, s = 'on'; else, s = 'off'; end;
emg = uicontrol(cfg, ...
	'Position', [120 height-85 50 20], ...
	'Style', 'checkbox', ...
	'String', 'EMG', ...
	'Enable', s, ...
	'Value', gotEMG);
	
% OK, cancel buttons
uicontrol(cfg, ...		% buttons
	'Position',[width/2-70 15 60 25], ...
	'String','OK', ...
	'Callback','set(gcbf,''UserData'',1);uiresume');
uicontrol(cfg, ...
	'Position',[width/2+10 15 60 25], ...
	'String','Cancel', ...
	'Callback','uiresume');

% wait for input
uiwait(cfg);

% process response
if get(cfg, 'UserData'),
	choix = struct('INDEX', get(list, 'Value'), ...
					'SPEECH', get(speech, 'Value'), ...
					'VPS', get(vps, 'Value'), ...
					'EMG', get(emg, 'Value'));
else,
	choix = [];
end;
delete(cfg);
