function [dataSet, setName] = LoadMBeamSet(setName)
%LOADMBEAMSET  - load trajectory dataset (U. Wisc. Microbeam file format)
%
% 	usage:  [dataSet, setName] = LoadMBeamSet(setName)
%
% where SETNAME determines the common set filename filter
% if SETNAME contains the '*' character it is interpreted as a mask
% if empty (no argument) selected using interactive dialog box
%
% expects to find files SETNAME.wav (audio, MS WAV format),
%	SETNAME.txy (pellet data, non-differential format)
%
% returns DATASET, an array of structs with fields
%	NAME	- trajectory name
%	SRATE	- sampling rate (Hz)
%	SIGNAL	- signal data [nSamp (samples) x nComp (components)]
%

% mkt 09/01


%	constants

pelExt = '.TXY';		% pellet data extension
audExt = '.WAV';		% audio data extension
names = {'UL','LL','T1','T2','T3','T4','MNI','MNM'};
mask = ['*' pelExt];
misTrack = 1e6;			% flags mistracking (replaced by NaN)


%	parse args

if nargin < 1, setName = []; end;
if findstr(setName, '*'), mask = setName; setName = []; end;

%	get setName interactively

if isempty(setName),
	[fileName, pathName] = uigetfile(mask, 'Select mBeam dataset');
	if fileName == 0, dataSet = []; return; end;
	setName = [pathName fileName];
end;
[pathName, setName] = fileparts(setName);
if pathName(end) ~= filesep, pathName(end+1) = filesep; end;

%	load acoustic data

a = ReadAudio([pathName , setName , audExt]);
dataSet = struct('NAME','AUDIO', 'SRATE',a.SRATE, 'SIGNAL',double(a));

%	load pellet data

fid = fopen([pathName , setName , pelExt], 'rt');
if fid == -1,
	fprintf('Error attempting to open pellet data file %s\n', [pathName , setName , pelExt]);
	dataSet = [];
	return;
end;

% format is offset; PX,PY for all P in NAMES
pd = fscanf(fid, '%d %d %d %d %d %d %d %d %d %d %d %d %d %d %d %d %d\n', [17 inf])';
fclose(fid);
t = pd(:,1);						% offsets (microsecs)
t = sort(diff(t));
sr = 1e6 / t(round(length(t)/2));	% use middle (majority) difference for sRate
pd = pd(:,2:end);
pd(find(pd == misTrack)) = NaN;		% find mistrackings
pd = pd ./ 1000;					% microns -> mm

for n = 1 : length(names),
	dataSet(n+1).NAME = names{n};
	dataSet(n+1).SRATE = sr;
	dataSet(n+1).SIGNAL = pd(:,2*n-1:2*n);
end;
