function [dataSet, setName] = LoadNewMBeamSet(setName, headCor, angle, doFilt, cutoff, names)
%LOADNEWMBEAMSET  - load trajectory dataset (U. Wisc. Microbeam new file format)
%
% 	usage:  [dataSet, setName] = LoadNewMBeamSet(setName, headCor, angle, doFilt, cutoff, names)
%
% where SETNAME determines the common set filename filter
% (if empty selected using interactive dialog box)
%
% does head correction if HEADCOR is nonzero (default)
% uses rotation ANGLE if specified (in degrees), else mean of reference angle (maxI:maxN) over dataset
% does Butterworth low-pass filtering @ CUTOFF Hz if DOFILT is non-zero (default)
% NAMES is a cellstr array of reference channel names
%	(default = {'maxN' 'maxG' 'maxI'}; use this order for replacements)
%
% returns DATASET, an array of structs with fields
%	NAME	- trajectory name
%	SRATE	- sampling rate (Hz)
%	SIGNAL	- signal data [nSamp (samples) x nComp (components)]
%	LABELS	- flags mistracked samples
%	EXTINFO	- a structure specifying extraction info with fields
%		SRCID	- set ID
%		HEADCOR	- head correction angle in degrees ([] if no head correction)
%		FILTER	- low-pass filtering cutoff in Hz ([] if no filtering)
%
% expects to find the following files within the same directory:
%	Pn		- physiological record info file
%	Pn.00	- audio
%	Rn		- X-ray channel info file
%	Rn.NN	- pellet trajectory for channel NN (%02d)
%
% Notes:  the sampling rates, sample counts, and durations given in the info files are
% not accurate and are ignored.  The durations are derived from the microsecond sample
% offset values for the pellet data and from an assumed 21739.13Hz sampling rate for
% audio data.  Pellets are named and ordered from the Rn info file.  Data are resampled
% for isochronous alignment.  Mistracked samples are replaced with initial sample values
% and their msec offsets are defined as labels.

% 07/98  mkt


%	constants

order = 'b';			% byte order
maxN = 'maxN';			% reference channel names (order sensitive!)
maxG = 'maxG';
maxI = 'maxI';
audioSR = 21739.13;		% audio sampling rate
OOB = 1e6;				% flags mistracking

%	parse args

if nargin < 1, setName = []; end;
if nargin < 2 | isempty(headCor), headCor = 1; end;
if nargin < 3, angle = []; end;
if nargin < 4 | isempty(doFilt), doFilt = 1; end;
if nargin < 5 | isempty(cutoff), cutoff = 10; end;		% default cutoff 10 Hz
if nargin == 6,
	if iscellstr(names) & length(names)==3,
		maxN = names{1};
		maxG = names{2};
		maxI = names{3};
	else,
		error('Expecting length 3 cellstring array for NAMES argument');
	end;
end;


%	get setName interactively

if isempty(setName),
	[fileName, pathName] = uigetfile('R*', 'Select dataset info file');
	if fileName == 0, dataSet = []; return; end;
	setName = fileName(2:end);

%	split pathName from setName

else,
	switch computer,				% find system delimiter (VMS not supported!)
		case 'MAC2', delim = ':';
		case 'PCWIN', delim = '\';
		otherwise, delim = '/';
	end;
	i = findstr(setName, delim);
	if isempty(i),
		pathName = [];
	else,
		pathName = setName(1:i(end));
		setName = setName(i(end)+1:end);
	end;
end;

%	parse fileset

files = dir([pathName, '*', setName, '*']);
if length(files) == 0,
	error(['no files found matching ', pathName, setName, '*']);
end;
files = {files.name}';					% files matching mask
uFiles = cellstr(upper(char(files)));	% upper case for testing

if strcmp(uFiles{1},['P' setName]),
	pinfo = files{1};					% physiological info file
else,
	error(['missing physiological info file (P' setName ')']);
end;

if strcmp(uFiles{2},['P' setName '.00']),
	audio = files{2};					% audio file
else,
	error(['missing audio file (P' setName '.00)']);
end;

i = strmatch('R', uFiles);
if isempty(i),
	error(['missing X-ray record info file (R' setName ')']);
else,
	xinfo = files{i(1)};
end;

i = strmatch('X', uFiles);
if isempty(i),
	error(['missing X-ray pellet trajectory files (X' setName '.*)']);
else,
	xFiles = files(i);
end;
srcID = str2num(setName);				% set ID

%	load audio data

fp = fopen([pathName pinfo], 'rt');			% read info file
a = fscanf(fp,'%s');
fclose(fp);
i = findstr(a,'Chan:');
j = findstr(a,'File:');
an = a(i+5:j-1);							% audio channel name
dataSet = struct('NAME', an, 'SRATE', audioSR, 'SIGNAL', [], 'SCALING', [], 'LABELS', [], 'EXTINFO', struct('SRCID',srcID));
fp = fopen([pathName audio], 'rb', order);	% read audio data
dataSet.SIGNAL = fread(fp, inf, 'int16');
fclose(fp);


%	parse X-ray records

fp = fopen([pathName xinfo], 'rt');
while 1,
	a = fgets(fp);
	if isnumeric(a), break; end;
	
	if ~isempty(findstr(a, 'Task:')),		% retrieve setname (Task field)
		i = findstr(a, ':');
		setName = sscanf(a(i(1):end),'%*c %s');
		n = double(upper(setName));
		i = ~((n>=double('0') & n<=double('9')) | (n>=double('A') & n<=double('Z')));
		setName(find(i)) = '_';				% zap gremlins
	end;
	
	if ~isempty(findstr(a, 'Pellet:')),		% extract channel info
		n = sscanf(a,'%*s %s');				% channel name
		a = fgets(fp);						% File field
		fn = upper(sscanf(a,'%*s %s'));
		dataSet(end+1) = struct('NAME', n, 'SRATE', [], 'SIGNAL', [], 'SCALING', [], 'LABELS', [], 'EXTINFO', struct('SRCID',srcID,'HEADCOR',[],'FILTER',[]));
		[fid, msg] = fopen([pathName fn], 'rb', order);
		if fid == -1,
			fclose(fp);
			error([msg, ' (', fn, ')']);
		end;
		dataSet(end).SIGNAL = fread(fid, [3, inf], 'int32')';	% X, Y (microns); time (microsecs)
		fclose(fid);
%	dataSet(end).SRATE = 1e6*size(dataSet(end).SIGNAL,1)/diff(dataSet(end).SIGNAL([1 end],3));
		switch n,
			case {maxN}, maxN = length(dataSet);	% remember ref channel indices
			case {maxG}, maxG = length(dataSet);
			case {maxI}, maxI = length(dataSet);
			otherwise,
		end;
	end;
	
end;
fclose(fp);

if length(dataSet) ~= length(xFiles)+1,
	error(sprintf('expected %d files but loaded %d\n', length(xFiles)+1, length(dataSet)));
end;


% 	find latest onset, earliest termination in msecs

timing = [0 1e6*size(dataSet(1).SIGNAL,1)/audioSR];		% audio duration (microsecs)					
for t = 2 : length(dataSet),				
	timing(1) = max([timing(1) dataSet(t).SIGNAL(1,3)]);
	timing(2) = min([timing(2) dataSet(t).SIGNAL(end,3)]);
end;
timing = timing / 1000;

%	do time alignment

% audio
dataSet(1).SIGNAL = dataSet(1).SIGNAL(round(timing(1)*audioSR/1000)+1 : round(timing(2)*audioSR/1000));

maxSamps = length(dataSet(1).SIGNAL);
for t = 2 : length(dataSet),
	data = dataSet(t).SIGNAL;
	
% replace mistracks with initial sample values; define as labels
	misTrack = find(data(:,1)==OOB);
	if ~isempty(misTrack),
		data(misTrack,1) = data(1,1);
		data(misTrack,2) = data(1,2);
		misTrack = data(misTrack,3)./1000 - timing(1);		% msec offsets of mistracked samples (time-aligned)
		lab(length(misTrack)) = struct('NAME', [], 'OFFSET', [], 'NOTE', [], 'VALUE', []);
		for i = 1 : length(misTrack),
			lab(i).OFFSET = misTrack(i);
			lab(i).NOTE = 'mistracked sample';
		end;
		dataSet(t).LABELS = lab;
	end;

% upsample; microns --> mm
	xi = [data(1,3) : 1000 : data(end,3)]';		% --> 1000Hz
	signal = [interp1(data(:,3), data(:,1), xi, 'cubic') , interp1(data(:,3), data(:,2), xi, 'cubic')] ./ 1000;

% time align
	i = round(timing(1) - data(1,3)/1000)+1;	% starting, ending sample
	j = round(timing(2) - data(1,3)/1000);

% decimate
	dataSet(t).SIGNAL = [decimate(signal(i:j,1),10) decimate(signal(i:j,2),10)];
	dataSet(t).SRATE = 100;						% --> 100Hz
	maxSamps = min([maxSamps length(dataSet(t).SIGNAL)]);				
end;


%	force consistent data lengths

for t = 2 : length(dataSet),
	dataSet(t).SIGNAL = dataSet(t).SIGNAL(1:maxSamps,:);
end;

%	do head correction if necessary

if headCor,

	if ~(isnumeric(maxN) & isnumeric(maxG) & isnumeric(maxI)),
		error('missing reference channel data');
	end;

	dataSet(maxN).SIGNAL = dataSet(maxN).SIGNAL - dataSet(maxI).SIGNAL;		% nose offset correction
	if isempty(angle),		% use mean angle if unspecified
		basis = mean(cart2pol(dataSet(maxN).SIGNAL(:,1), dataSet(maxN).SIGNAL(:,2)));
		angle = basis * 180 / pi;
		disp(['LoadNewMBeamSet:  correction angle = ', num2str(angle)]);
	else,
		basis = angle * pi / 180;	% to radians
	end;
	rotAngle = cart2pol(dataSet(maxN).SIGNAL(:,1), dataSet(maxN).SIGNAL(:,2)) - basis;		% deviations from basis
	for t = 2 : length(dataSet),
		if ~any(t == [maxN maxI maxG]),

% translation:  subtract offset to maxilla
			dataSet(t).SIGNAL = dataSet(t).SIGNAL - dataSet(maxI).SIGNAL;

% rotation:  align maxilla - nose angle with reference angle
			[theta, r] = cart2pol(dataSet(t).SIGNAL(:,1), dataSet(t).SIGNAL(:,2));
			[x, y] = pol2cart(theta - rotAngle, r);
			dataSet(t).SIGNAL = [x y];
			dataSet(t).EXTINFO.HEADCOR = angle;
			if doFilt, dataSet(t).EXTINFO.FILTER = cutoff; end;
		end;
	end;

% delete reference channels
	dataSet([maxN maxI maxG]) = [];

end;


%	filter

if doFilt, 
	dataSet = FilterTrajData(dataSet, cutoff);
end;


%	set scaling

range = ceil(max(abs([min(dataSet(1).SIGNAL) max(dataSet(1).SIGNAL)])));
dataSet(1).SCALING = struct('TYPE', 'UNIQUE', 'RANGE', [-range range]);

for t = 2 : length(dataSet),		% non-audio
	dataSet(t).SCALING = struct('TYPE', 'COMMON', 'RANGE', ...
				[min(dataSet(t).SIGNAL)' max(dataSet(t).SIGNAL)']);
end;

