function [dataSet, setName] = LoadPCMSet(setName, specifier)
%LOADPCMSET  - load trajectory dataset (PCM file format)
%
% 	usage:  [dataSet, setName] = LoadPCMSet(setName, specifier)
%
% where SETNAME determines the common set filename filter
% (if empty selected using interactive dialog box)
%
% optional SPECIFIER argument is a trajectory name common
%  to all sets, used to delimit setName (default AUDIO)
%
% returns DATASET, an array of structs with fields
%	NAME	- trajectory name
%	SRATE	- sampling rate (Hz)
%	SIGNAL	- signal data [nSamp (samples) x nComp (components)]
%				where recognized components are X Y Z

% v1.5 02/03/98  mark tiede
% Copyright (c) 1998 ATR Human Information Processing Labs, Inc.

%	constants

defSpec = 'audio';		% default specifier
ext = '.pcm';			% filename extension
comps = 'XYZxyz';		% recognized components


%	parse args

if nargin < 1, setName = []; end;
if nargin < 2, specifier = defSpec; end;


%	get setName interactively

if isempty(setName),
	[fileName, pathName] = uigetfile(['*', specifier, ext], 'Select dataset member');
	if fileName == 0, dataSet = []; return; end;
	i = findstr(fileName, specifier);
	setName = fileName(1:i(1)-1);

%	split pathName from setName

else,
	switch computer,				% find system delimiter (VMS not supported!)
		case 'MAC2', delim = ':';
		case 'PCWIN', delim = '\';
		otherwise, delim = '/';
	end;
	i = findstr(setName, delim);
	if isempty(i),
		pathName = [];
	else,
		pathName = setName(1:i(end));
		setName = setName(i(end)+1:end);
	end;
end;


%	get files matching mask

files = dir([pathName, setName, '*', ext]);
if length(files) == 0,
	error(['no files found matching ', pathName, setName, '*', ext]);
end;


%	load files

ti = 0;							% trajectory index
fi = 1;							% file index
while fi <= length(files),

	fileName = files(fi).name;
	trajName = fileName(length(setName)+1:end-length(ext));

%	load single component data (audio, emg, ...)

	if isempty(findstr(comps, trajName(end))),
		[signal, sRate] = PCMread([pathName, fileName]);
		fi = fi + 1;

%	load multiple component data

	else,
		trajName = trajName(1:end-1);
		ci = 0;					% component index
		while fi<=length(files) & ~isempty(findstr(fileName, trajName)),
			if ~isempty(findstr(comps, fileName(end-length(ext)))),
				ci = ci + 1;
				[signal(:, ci), sRate] = PCMread([pathName, fileName]);
			end;
			fi = fi + 1;
			if fi <= length(files), fileName = files(fi).name; end;
		end;
	end;	

%	insert into output structure

	trajName = trajName(find(trajName ~= '_'));		% kill underscore chars
	ti = ti + 1;
	dataSet(ti) = struct('NAME', trajName, ...
							'SRATE', sRate, ...
							'SIGNAL', signal);
	clear signal;
end;


%	kill setName trailing underscore

if setName(end) == '_',
	setName = setName(1:end-1);
end;
