function [dataSet, setName, cfg, pal, img, pha] = m_load(format, dataSet, setName)
%M_LOAD  - MAVIS dataset loader
%
% This function provides the interface between MAVIS and alternative
% user-specified format data; tailor as appropriate.
%
% 	FORMAT	- string specifying format type to import; currently supported
%		EVB		- EVB's consolidated datasets
%		MBEAM	- U. Wisconsin microbeam datasets
%		NEWMB	- new U.W. microbeam format datasets
%		PCM		- Haskins format PCM datasets
%		MAT		- multi-dataset MAT files
%		NATIVE	- MAVIS format dataset
%
% returns DATASET, an array of structs with fields
%	NAME	- trajectory name
%	SRATE	- sampling rate (Hz)
%	SIGNAL	- signal data [nSamp (samples) x nComp (components)]
%				where components are trajectory dimensions (X, Y, ...)
%	LABELS	- trajectory labelling info
%	SCALING	- structure with fields
%		TYPE	- either 'COMMON' (data pooled for display scaling), 
%				   or 'UNIQUE' (data scaled separately in specified range)
%		RANGE	- signal scaling bounds [nComps x (min, max)]

% v1.5 02/03/98  mark tiede
% Copyright (c) 1998 ATR Human Information Processing Labs, Inc.

if nargin < 2, dataSet = []; end;
if nargin < 3, setName = []; end;
cfg = [];
pal = [];
img = [];
pha = [];

%	branch by supported type

switch format,

% EVB data
	case 'EVB',
		[dataSet, setName] = LoadEVBSet;

% Microbeam data
	case 'MBEAM',
		[headCor, angle, doFilt, cutoff] = CfgMBeam;
		if isempty(headCor), return; end;		% cancel
		[dataSet, setName] = LoadMBeamSet([],[], headCor, angle, doFilt, cutoff);
		
% New Microbeam data
	case 'NEWMB',
		[headCor, angle, doFilt, cutoff] = CfgMBeam;
		if isempty(headCor), return; end;		% cancel
		[dataSet, setName] = LoadNewMBeamSet([], headCor, angle, doFilt, cutoff);
		
% Haskins PCM data
	case 'PCM',
		[dataSet, setName] = LoadPCMSet;

% MAT file variable
	case 'MAT',
		[file, pathName] = uigetfile('*.mat', 'Select MAT file');
		if file == 0, return; end;			% if cancelled
		fileName = [pathName file];	
		vars = who('-file', fileName);		% variables in SETNAME.MAT
		i = GetVar(vars);					% get selected variable
		if isempty(i), return; end;			% cancel
		name = char(vars(i));
		load(fileName, name);				% load specified variable
		eval(['dataSet=',name,';']);
		setName = char(vars(i));
		if ~Valid(dataSet),
			dataSet = [];
			set(warndlg([setName, ' is not a valid MAVIS format dataset'], ''), 'WindowStyle', 'modal');
			return;
		end;

% MAVIS native format (dataset variable matches filename)
	case 'NATIVE',
	
	% if DATASET is a struct, test for valid MAVIS format data
		if ~isempty(dataSet),
			if ~Valid(dataSet),
				dataSet = [];
				set(warndlg([setName, ' is not a valid MAVIS format dataset'], ''), 'WindowStyle', 'modal');
				return;
			end;
			dataSet = AddFields(dataSet);
			return;
		end;

	% get the file interactively
		[file, pathName] = uigetfile('*.mat', 'Select MAVIS format dataset');
		if file == 0, return; end;	% if cancelled
		fileName = [pathName file];	
		setName = file;
		i = findstr(setName, '.');
		if ~isempty(i), setName = setName(1:i(1)-1); end;	

	% load data from variable named SETNAME in SETNAME.MAT file
		vars = who('-file', fileName);		% variables in SETNAME.MAT
		if isempty(strmatch(setName, char(vars), 'exact')),
			set(warndlg(['File ', file, ' does not contain a variable called ', setName, ' (use IMPORT)'], ''), 'WindowStyle', 'modal');
			return;
		end;
		load(fileName);		% load all data in SETNAME.MAT (including CFG, PAL, IMG, PHA if present)
		eval(['dataSet = ', setName, ';']);
		if ~Valid(dataSet),
			dataSet = [];
			set(warndlg([setName, ' is not a valid MAVIS format dataset'], ''), 'WindowStyle', 'modal');
			return;
		end;

% error
	otherwise,
		error([format, ' is not a currently supported import format']);
end;

dataSet = AddFields(dataSet);		% add LABELS, SCALING fields if necessary


%=============================================================================
% ADDFIELDS  - add SCALING, LABELS fields if necessary

function mavisDataSet = AddFields(dataSet)

fields = char(fieldnames(dataSet));
needScaling = isempty(strmatch('SCALING', fields, 'exact'));
needLabels = isempty(strmatch('LABELS', fields, 'exact'));
if ~(needScaling | needLabels),
	mavisDataSet = dataSet;			% nothing to do
	return;
end;

%	add required fields
for t = 1 : length(dataSet),
	clear element;
	element = dataSet(t);
	
% compute scaling
	if needScaling,
		nComp = size(element.SIGNAL,2);		% number of trajectory components

% monodimensional signals set to UNIQUE bipolar scaling
		if nComp == 1,
			minMax = [min(element.SIGNAL) max(element.SIGNAL)];
			element.SCALING = struct('TYPE', 'UNIQUE', 'RANGE', minMax);
% 			if max(abs(minMax))>2048,	% 16bit (+/-32768)
% 				element.SCALING = struct('TYPE', 'UNIQUE', 'RANGE', [-32768 32768]);
% 			else,						% 12bit (+/-2048)
% 				element.SCALING = struct('TYPE', 'UNIQUE', 'RANGE', [-2048 2048]);
% 			end;
			
% multidimensional signals use COMMON (pooled) scaling
		else,
			element.SCALING = struct('TYPE', 'COMMON', 'RANGE', ...
				[min(element.SIGNAL)' max(element.SIGNAL)']);
		end;
	end;

% add empty labels field	
	if needLabels,
		element.LABELS = [];
	end;
	
	mavisDataSet(t) = element;
end;


%=============================================================================
% CFGMBEAM  - configure Microbeam loading

function [headCor, angle, doFilt, cutoff] = CfgMBeam

figPos = get(0, 'ScreenSize');
if strncmp(computer, 'SGI', 3), as = 'left'; else, as = 'right'; end;	% SGI bug:  doesn't like right alignment

width = 275;
height = 120;
figPos = [figPos(1)+(figPos(3)-width)/2, figPos(2)+(figPos(4)-height)/2, width, height];

cfg = dialog('Name', 'Configure Loading', ...
	'Position', figPos, ...
	'ButtonDownFcn', '', ...
	'UserData', 0);

headCor = uicontrol(cfg, ...	% headCor
	'Position',[20 91 100 20], ...
	'String','Head Correction', ...
	'Value', 1, ...
	'Style','checkbox');
uicontrol(cfg, ...				
	'Position', [130 90 40 17], ...
	'HorizontalAlignment', as, ...
	'String','Angle:', ...
	'Style','text');
angle = uicontrol(cfg, ...
	'Style', 'edit', ...
	'HorizontalAlignment', 'left', ...
	'String', '(use mean)', ...
	'Position', [175 90 80 20]);
	
doFilt = uicontrol(cfg, ...	% filter
	'Position',[20 61 100 20], ...
	'String','Low-pass filter', ...
	'Value', 1, ...
	'Style','checkbox');
uicontrol(cfg, ...				
	'Position', [130 60 40 17], ...
	'HorizontalAlignment', as, ...
	'String','Cutoff:', ...
	'Style','text');
cutoff = uicontrol(cfg, ...
	'Style', 'edit', ...
	'HorizontalAlignment', 'left', ...
	'String', '10', ...
	'Position', [175 60 45 20]);
uicontrol(cfg, ...				
	'Position', [225 60 40 17], ...
	'HorizontalAlignment', 'left', ...
	'String','(Hz)', ...
	'Style','text');
	
% OK, cancel buttons
uicontrol(cfg, ...		% buttons
	'Position',[width/2-70 15 60 25], ...
	'String','OK', ...
	'Callback','set(gcbf,''UserData'',1);uiresume;');
uicontrol(cfg, ...
	'Position',[width/2+10 15 60 25], ...
	'String','Cancel', ...
	'Callback','uiresume;');

% wait for input
uiwait(cfg);

% process response
if get(cfg,'UserData'),
	headCor = get(headCor, 'Value');
	angle = str2num(get(angle, 'String'));
	doFilt = get(doFilt, 'Value');
	cutoff = str2num(get(cutoff, 'String'));
else,
	headCor = [];
	angle = [];
	doFilt = [];
	cutoff = [];
end;
delete(cfg);


%=============================================================================
% GETVAR  - return index of selected variable within vars list

function index = GetVar(vars)

figPos = get(0, 'ScreenSize');

width = 180;
height = 125;
figPos = [figPos(1)+(figPos(3)-width)/2, figPos(2)+(figPos(4)-height)/2, width, height];

cfg = dialog('Name', 'Select Variable', ...
	'Position', figPos, ...
	'ButtonDownFcn', '', ...
	'UserData', 0);

% variable list
list = uicontrol(cfg, ...
	'Position',[(width-150)/2 height-65 150 56], ...
	'String', vars, ...	
	'ListBoxTop', 1, ...
	'Style','listbox', ...
	'Value', 1, ...
	'Callback', 'if strcmp(get(gcbf,''SelectionType''),''open''),set(gcbf,''UserData'',1);uiresume;end');
	
% OK, cancel buttons
uicontrol(cfg, ...		% buttons
	'Position',[width/2-70 15 60 25], ...
	'String','OK', ...
	'Callback','set(gcbf,''UserData'',1);uiresume');
uicontrol(cfg, ...
	'Position',[width/2+10 15 60 25], ...
	'String','Cancel', ...
	'Callback','uiresume');

% wait for input
uiwait(cfg);

% process response
if get(cfg, 'UserData'),
	index = get(list, 'Value');
else,
	index = [];
end;
delete(cfg);


%=============================================================================
% VALID  - verify dataSet format

function isValid = Valid(dataSet)

isValid = isstruct(dataSet);
if isValid,
	fields = char(fieldnames(dataSet));
	reqFields = {'NAME' 'SRATE' 'SIGNAL'};		% required fields
	for i = 1 : length(reqFields),
		if isempty(strmatch(char(reqFields(i)), fields, 'exact')), isValid = 0; end;
	end;
end;


