function m_out = mavis(varargin)
%MAVIS  - Multiple Articulator Visualization tool
%
% MAVIS is a group of integrated MATLAB functions useful for viewing
% synchronized movement data such as that generated by point source
% tracking methods like the magnetometer (EMMA), OPTOTRAK, or X-ray
% microbeam systems.  It provides simultaneous display of time varying
% trajectories and their spatial distribution at a temporal offset, which
% can be incremented automatically for various animation effects.
%
% For extended help on MAVIS, start the GUI by entering MAVIS at the
% command line, then select the ABOUT option from the MAVIS menu.
%
% The following is a summary of MAVIS command line options:
%
%          mavis        - initialize GUI; pop windows foremost if active
%
%          mavis(data)  - initialize & display data
%
%          mavis('Name1', 'Value1', ..., 'NameN', 'ValueN')
%             where NAME:VALUE pairs specify one of
%                    CONFIG   - configuration data
%                    DATA     - dataset
%                    IMAGE    - image data
%                    LABELS   - (melba-generated) label data
%                    PALATE   - palate trace data
%
%   data = mavis('GETDATA')  - export (labelled) dataset
%   sel  = mavis('GETSEL')   - export current selection as new dataset
%   cfg  = mavis('GETCFG')   - export current configuration
%   labs = mavis('GETLAB')   - export labels
%
% One additional option is supported:  in case some lurking glitch prevents
% closing MAVIS normally, entering
%
%   mavis ABORT
%
% should successfully slay the monster.

% v1.6 24/10/01  by mark tiede

%	parse args

switch length(varargin),
	case 0,
		m_gui('INIT');
	case 1,
		if ischar(varargin{1}),
			switch upper(varargin{1}),
				case 'ABORT',		% force exit (debugging)
					m_gui('ABORT');
				case 'GETCFG',
					m_out = m_gui('GETCFG');
				case 'GETDATA',
					m_out = m_gui('GETDATA', 0);
				case 'GETLAB',
					m_out = m_gui('GETLAB');
				case 'GETSEL',
					m_out = m_gui('GETDATA', 1);
				otherwise,
					error(['MAVIS:  unrecognized keyword (', varargin{1}, ')']);
			end;
		else,
			m_gui('INIT');
			m_gui('LOADSET', varargin{1}, inputname(1));
		end;
	otherwise,
		if rem(length(varargin),2),
			error('Multiple arguments must be specfied as valid NAME : VALUE pairs');
		end;
		m_gui('INIT');
		cfgIndex = 0;
		for i = 1 : 2 : length(varargin),
			if ~ischar(varargin{i}), error('Invalid NAME : VALUE pair'); end;
			switch upper(varargin{i}),
				case 'CONFIG',
					cfgIndex = i+1;			% load config last
				case 'DATA',
					m_gui('LOADSET', varargin{i+1}, inputname(i+1));
				case 'IMAGE',
					m_gui('LOADIMG', varargin{i+1}, inputname(i+1));
				case 'LABELS',
					m_gui('LOADLBL', varargin{i+1}, inputname(i+1));
				case 'PALATE',
					m_gui('LOADPAL', varargin{i+1}, inputname(i+1));
				otherwise,
					error(['MAVIS:  unrecognized keyword (', varargin{i}, ')']);
			end;
		end;
		if cfgIndex > 0, m_gui('LOADCFG', varargin{cfgIndex}); end;
end;
