function m_out = ml_extract(action, aux, aux2, aux3)
%ML_EXTRACT  - MAVIS user labelling procedure
%
% Support for extraction based on label pairs:  boundary 1 is set when the mouse 
% is first clicked, boundary 2 is set upon release.  Exports selection between labels
% as a new variable (using label name for created variable name)

% 04/00 mkt

%	branch by action...

switch action,
		
%-----------------------------------------------------------------------------
% CONFIG:  handle configuration
%
% no configuration necessary, just show blurb

	case 'CONFIG',
		ShowBlurb;
		m_out = 1;					% report OK
		return;
		
		
%-----------------------------------------------------------------------------
% DOWN:  handle mouseDown
%
%	AUX		- internal state
%	AUX2	- index of clicked trajectory (into state.DATA)

	case 'DOWN',
		state = aux;
		saveAnnotate = state.LABEL.ANNOTATE;		% get current annotation status
		state.LABEL.ANNOTATE = 0;					% force it off for label creation
		set(state.GUI, 'UserData', state);

		label = struct('NAME', '', ...				% make label at click loc
					'OFFSET', [], ...				% (cursor offset)
					'NOTE', [], ...					% flags half initialized
					'VALUE', []);
		m_label('MAKE', label);						% create, display the label quietly

		state = get(state.GUI, 'UserData');			% retrieve internal state
		state.LABEL.ANNOTATE = saveAnnotate;		% restore annotation status
		set(state.GUI, 'UserData', state);

		set(state.TRAJWIN,  ...			% init motion handlers
			'WindowButtonMotionFcn', 'm_label MOVE;', ...
			'WindowButtonUpFcn', 'ml_extract UP;');


%-----------------------------------------------------------------------------
% PLOT:  plot label into the current axes
%
%	AUX		- x offset (in trajectory sample units) of label
%	AUX2	- yLim of current axis
%	AUX3	- label
%
% labels with an empty or non-numeric NOTE field are plotted as normal labels
% otherwise the NOTE field is assumed to hold one half of the extent

	case 'PLOT',
		x = aux;
		y = aux2 * .9;
		label = aux3;
		line([x x], y);					% this plots the OFFSET value
		
		if ~isempty(label.NOTE) & isnumeric(label.NOTE),
			x2 = label.NOTE*(x-1)/label.OFFSET + 1;	% cvt to samples
			line([x2 x2], y);			% this plots the NOTE value
			line([x x2], [y(2) y(2)], 'LineStyle', '--');	% join the tops
			if x < x2, x = x2; end;		% put the name following temporally later line
		end;
		
		if ~isempty(label.NAME),
			text(x, y(2), [' ', label.NAME], ...
				'VerticalAlignment', 'top', ...
				'fontname', 'geneva', ...
				'fontsize', 9);
		end;
		

%-----------------------------------------------------------------------------
% UP:  handle mouseUp

	case 'UP',
		set(gcbf, ...						% clear motion handlers
			'WindowButtonMotionFcn', '', ...
			'WindowButtonUpFcn', '');
		
		trajState = get(gcbf, 'UserData');
		state = get(trajState.GUI, 'UserData');
		t = trajState.TRACK;				% index of clicked trajectory
		
		downClickLabel = state.DATA(t).LABELS(end);
		state.DATA(t).LABELS(end) = [];		% delete the downClick label from the stored list
		set(trajState.GUI, 'UserData', state);
		label = struct('NAME', '', ...		% no name
					'OFFSET', [], ...		% use current cursor offset
					'NOTE', downClickLabel.OFFSET, ...
					'VALUE', []);
		
% create the label (m_label('MAKE') returns 1 on success, 0 on cancel)
		if m_label('MAKE', label),	
			state = get(trajState.GUI, 'UserData');
			
% put the extent head into label.OFFSET, extent tail into label.NOTE
			if downClickLabel.OFFSET > state.DATA(t).LABELS(end).OFFSET,
				state.DATA(t).LABELS(end).NOTE = downClickLabel.OFFSET;
				state.DATA(t).LABELS(end).VALUE = [state.DATA(t).LABELS(end).VALUE ; downClickLabel.VALUE];
			else,
				state.DATA(t).LABELS(end).NOTE = state.DATA(t).LABELS(end).OFFSET;
				state.DATA(t).LABELS(end).OFFSET = downClickLabel.OFFSET;
				state.DATA(t).LABELS(end).VALUE = [downClickLabel.VALUE ; state.DATA(t).LABELS(end).VALUE];
			end;
			set(trajState.GUI, 'UserData', state);

% export selection to workspace
			global msecsToSamps;
			head = state.DATA(t).LABELS(end).OFFSET;
			tail = state.DATA(t).LABELS(end).NOTE;
			data = state.DATA;
			for i = 1 : length(data),
				hs = msecsToSamps(head, data(i).SRATE);
				ts = msecsToSamps(tail, data(i).SRATE);
				data(i).SIGNAL = data(i).SIGNAL(hs:ts,:);
				if strcmp(data(i).SCALING.TYPE, 'COMMON'),		% recompute scaling range
					data(i).SCALING.RANGE = [min(data(i).SIGNAL)' max(data(i).SIGNAL)'];
				end;
				data(i).LABELS = [];
			end;
			name = state.DATA(t).LABELS(end).NAME;
			assignin('base', name, data);
			evalin('base', name)		% for listing			

% label creation cancelled
		else,
			m_traj('INIT', state, 1);		% refresh
		end;

	
%-----------------------------------------------------------------------------
% error

	otherwise,
		error(['ML_EXTRACT:  unrecognized action (', action, ')']);
	
end;

%=============================================================================
% SHOWBLURB

function ShowBlurb

figPos = get(0, 'ScreenSize');
width = 300;
height = 150;
figPos = [figPos(1)+(figPos(3)-width)/2, figPos(2)+(figPos(4)-height)/2, width, height];

cfg = dialog('Name', 'ML_EXTRACT', ...
	'Position', figPos, ...
	'ButtonDownFcn', '');

blurb = ['Support for extraction based on label pairs:  boundary 1 is set when the mouse ' ...
		'is first clicked, boundary 2 is set upon release.  Exports selection between labels ' ...
		'as a new variable (using label name for created variable name).'];

uicontrol(cfg, ...
	'Position', [10 60 width-20 80], ...
	'Style', 'frame');
uicontrol(cfg, ...
	'Position', [13 63 width-26 74], ...
	'Style', 'text', ...
	'HorizontalAlignment', 'left', ...
	'String', blurb);

% OK button
uicontrol(cfg, ...		% buttons
	'Position',[width/2-30 15 60 25], ...
	'String','OK', ...
	'Callback','uiresume');

% wait for input
uiwait(cfg);
delete(cfg);
