function m_out = ml_template(action, aux, aux2, aux3)
%ML_TEMPLATE  - MAVIS user labelling procedure
%
% This example template mimics default MAVIS labelling behavior, but 
% draws labels with dashed lines and attaches a user-configured NOTE 
% string.   Cursor movement is tracked while the mouse button is down 
% (see the MOVE handler), and the label is created on release (see UP).
%
% Clone & tailor this file to create new user labelling procedures.
%
% MAVIS user plotting procedures must supply three handlers:
%	CONFIG	- process any configuration necessary
%	DOWN	- mouse down handler
%	PLOT	- label plotting handler
%
% The selected user plotting routine is called by MAVIS whenever labels
% need to be plotted, the mouseDown handler upon a modified mouse click 
% in the trajectory display, and the config handler when the user proc
% is first selected.
%
% To create a label, user-handlers must at some point call
%
%	m_label('MAKE', label)
%
% where LABEL must be a struct with at least the following fields defined
%   NAME    - string specifying displayed label name
%   OFFSET  - label offset from time 0 in msecs
%   NOTE    - any additional information
%   VALUE   - (contents ignored)
%
% all of which may be empty.  If the OFFSET is empty m_label sets it to
% the current cursor offset.  If the NOTE field is a string it is displayed
% and may be edited, else if not empty it is stored in the form passed.
%
% Note that while user procedures can create labels with additional fields,
% such labels are incompatible with those created by the default handlers
% and cannot be mixed with them.

%	branch by action...

switch action,
		
%-----------------------------------------------------------------------------
% CONFIG:  handle configuration
%
%	AUX		- internal state
%
% returns M_OUT == 1  - OK
%               == 0  - cancel
%
% For a minimal handler just set m_out = 1 and return
% Here we get a default value to display for the label note and save it
% as the state.LABEL.USERDATA entry

	case 'CONFIG',
		state = aux;						% get internal state
		labelData = DoConfig(state.LABEL.USERDATA);
		if isempty(labelData),
			m_out = 0;				% report cancel
			return;
		end;
		state.LABEL.USERDATA = labelData;
		set(state.GUI, 'UserData', state);	% update internal state
		m_out = 1;							% report OK
		return;
		
		
%-----------------------------------------------------------------------------
% DOWN:  handle mouseDown
%
%	AUX		- internal state
%	AUX2	- index of clicked trajectory (into state.DATA)

	case 'DOWN',
		state = aux;
		set(state.TRAJWIN,  ...			% init motion handlers
			'WindowButtonMotionFcn', 'ml_template MOVE;', ...
			'WindowButtonUpFcn', 'ml_template UP;');


%-----------------------------------------------------------------------------
% MOVE:  cursor moved
%
% The same functionality can be obtained by setting the WindowButtonMotionFcn
% to 'm_label MOVE;' in the DOWN handler above

	case 'MOVE',
		global fixstr;
		trajState = get(gcbf, 'UserData');
		state = get(trajState.GUI, 'UserData');
	
% get the location of the cursor
		curPt = get(trajState.CURSORH, 'CurrentPoint');
		x = curPt(1,1);
		y = curPt(1,2);

% ignore if out of plotting bounds		
		xLim = get(trajState.CURSORH, 'XLim');			
		yLim = get(trajState.CURSORH, 'YLim');
		if (x < xLim(1)) | (x > xLim(2)) | (y < yLim(1)) | (y > yLim(2)), return; end;

% update GUI field and spatial display if necessary
		oldX = get(trajState.CURSORL, 'XData');
		if x ~= oldX(1),
			set(trajState.CURSORL, 'XData', [x x]);
			state.CURSOR = x;
			set(state.CURSF, 'String', fixstr(x));
			set(trajState.GUI, 'UserData', state);
			m_spat('UPDATE', state);
		end;
		

%-----------------------------------------------------------------------------
% PLOT:  plot label into the current axes
%
%	AUX		- x offset (in trajectory sample units) of label
%	AUX2	- yLim of current axis
%	AUX3	- label

	case 'PLOT',
		x = [aux aux];
		y = aux2 * .9;
		label = aux3;
		line(x, y, 'LineStyle', '--');
		if ~isempty(label.NAME),
			text(x(2), y(2), [' ', label.NAME], ...
				'VerticalAlignment', 'top', ...
				'fontname', 'geneva', ...
				'fontsize', 9);
		end;
		

%-----------------------------------------------------------------------------
% UP:  handle mouseUp

	case 'UP',
		set(gcbf, ...						% clear motion handlers
			'WindowButtonMotionFcn', '', ...
			'WindowButtonUpFcn', '');
		
		trajState = get(gcbf, 'UserData');	% retrieve stored labelData
		state = get(trajState.GUI, 'UserData');
		labelData = state.LABEL.USERDATA;

		label = struct('NAME', '', ...		% no name
					'OFFSET', [], ...		% use current cursor offset
					'NOTE', labelData.NOTE, ...	% set by CONFIG handler
					'VALUE', []);			% value contents ignored

		if m_label('MAKE', label),			% create the label; returns 1 on success, 0 on cancel
			t = trajState.TRACK;			% clicked trajectory
			state = get(trajState.GUI, 'UserData');	% refresh
			label = state.DATA(t).LABELS(end);		% new label is tail end of array
			if isempty(label.NAME), s = 'a label'; else, s = label.NAME; end;
			disp(['Created ', s, ' in ', state.DATA(t).NAME, ' at offset ', num2str(label.OFFSET), ' msecs']);
		end;
	
%-----------------------------------------------------------------------------
% error

	otherwise,
		error(['ML_TEMPLATE:  unrecognized action (', action, ')']);
	
end;

%=============================================================================
% DOCONFIG  - config handler
%
%   returns non-empty labelData on OK, empty result on cancel
%
% in this example we get a default value to display for the label note

function labelData = DoConfig(labelData)

figPos = get(0, 'ScreenSize');
width = 300;
height = 170;
figPos = [figPos(1)+(figPos(3)-width)/2, figPos(2)+(figPos(4)-height)/2, width, height];
if strncmp(computer, 'SGI', 3), as = 'left'; else, as = 'right'; end;	% SGI bug:  doesn't like right alignment

idString = 'ML_TEMPLATE';
if isempty(labelData) | ~strcmp(labelData.SOURCE, idString),		% if not previously configured
	labelData = struct('SOURCE', idString, ...
						'NOTE', 'made by ML_TEMPLATE()');
end;

cfg = dialog('Name', idString, ...
	'Position', figPos, ...
	'ButtonDownFcn', '', ...
	'UserData', 0);

% about
blurb = ['Mimics default MAVIS labelling behavior, but draws labels ' ...
		'with dashed lines and attaches a user-configured NOTE string.'];

uicontrol(cfg, ...
	'Position', [10 height-70 width-20 60], ...
	'Style', 'frame');
uicontrol(cfg, ...
	'Position', [13 height-67 width-26 54], ...
	'Style', 'text', ...
	'HorizontalAlignment', 'left', ...
	'String', blurb);

% editable text field and label
uicontrol(cfg, ...
	'Position', [10 60 110 17], ...
	'HorizontalAlignment', as, ...
	'String','Default note string:', ...
	'Style','text');
note = uicontrol(cfg, ...
	'Style', 'edit', ...
	'HorizontalAlignment', 'left', ...
	'String', labelData.NOTE, ...
	'Position', [125 60 150 20]);

% OK, cancel buttons
uicontrol(cfg, ...		% buttons
	'Position',[width/2-70 15 60 25], ...
	'String','OK', ...
	'Callback','set(gcbf,''UserData'',1);uiresume');
uicontrol(cfg, ...
	'Position',[width/2+10 15 60 25], ...
	'String','Cancel', ...
	'Callback','uiresume');

% wait for input
uiwait(cfg);
if get(cfg, 'UserData'),
	labelData.NOTE = get(note, 'String');
else,
	labelData = [];
end;
delete(cfg);
